
ENTITY = 'entity'
RETURN_TYPE = 'return_type'
PARAMETERS = 'parameters'
BODY = 'body'
METHODS = 'methods'
FIELDS = 'fields'

PARAMETER_TYPE_METHOD_INVOCATION_UNKNOWN = 'unknown'
#

#
NODE_TYPE_CLASS = 'class'
NODE_TYPE_METHOD = 'method'
NODE_TYPE_FIELD = 'field'
NODE_TYPE_MAPPER = 'mapper'
NODE_TYPE_MAPPER_SQL = 'sql'
NODE_TYPE_MAPPER_RESULT_MAP = 'resultMap'
NODE_TYPE_MAPPER_STATEMENT = 'statement'

DIFF_TYPE_CHANGED = 'changed'
DIFF_TYPE_IMPACTED = 'impacted'

JAVA_BASIC_TYPE = ['string', 'int', 'boolean', 'long', 'byte', 'short', 'float', 'double', 'char']
